package ASCII::Font;

=pod

=head1 NAME

ASCII::Font - utility routines for font handling 

=head1 SYNOPSIS

This module is not meant to be used directly.

=head1 DESCRIPTION

=cut

# -------------------------------------------------------------------

use strict;
use warnings;

use base 'Exporter';
our @EXPORT = qw();

use Carp qw( carp confess croak );
use FindBin;
use File::Basename;
use Font::TTF::Font;

use lib "$FindBin::RealBin";
use lib "$FindBin::RealBin/../lib";
use lib "$FindBin::RealBin/lib";

use ASCII::Debug;

our @metric_names = qw(neg_width global_descent pos_width global_ascent descent ascent advance_width right_bearing);

################################################################
# Find out the TTF name for a font, so that it can be used
# in SVG files
sub get_font_name {
    my $font_file   = shift;
    my $font_ttf    = Font::TTF::Font->open($font_file);
    confess "Could not initialize font to find out its name [$font_file]" unless $font_ttf;
    my ($font_name) = $font_ttf->{name}->read->find_name(6);
    confess "Could not determine font name [$font_file]" unless $font_name;
    return $font_name;
}

################################################################
# Collect metrics for font characters
sub get_metrics {
	my ($font_obj,$char) = @_;
	my %m;
	confess "Could not get metric using undefined font object ." unless defined $font_obj;
	confess "Could not get metric for undefined character."      unless defined $char;
	eval {
		@m{@metric_names} = $font_obj->bounding_box(string=>$char);
	};
	confess "Error fetching metrics for character [$char] [$@]" if $@;
	return %m;
}

sub make_font_obj {
    my ($font_file,$fg,$size) = @_;
    my $font = Imager::Font->new(file  => $font_file, color => $fg, size  => $size);
    confess "Could not create font from file [$font_file]" unless $font;
    return $font;
}


################################################################
# Assign each font to a group.
#
sub make_font_groups {
    my ($groups,@fonts) = @_;
    my %font_to_group;
    my @group_names = $groups ? sort {length($b) <=> length($a)} keys %$groups : ();
    my $def_group = -1;
    for my $font (@fonts) {
	if(! @group_names) {
	    $font_to_group{$font} = $def_group;
	} else {
	    my $found = 0;
	    for my $group_name (@group_names) {
		if($font =~ /$group_name/i) {
		    $font_to_group{$font} = $groups->{$group_name};
		    $found = 1;
		    last;
		}
	    }
	    $font_to_group{$font} = $def_group if ! $found;
	}
	printdebug(1,"font group",basename($font),$font_to_group{$font});
    }
    
    return %font_to_group;
}


# Get a list of font files from $CONF{input}{fontdir} that match $CONF{input}{fontrx}
#
# Prefer ttf over otf, if both versions exist
sub get_font_files {
    my ($dir,$rx_pass,$rx_fail) = @_;
    confess "Input font directory not defined" unless defined $dir;
    my @font_files;
    my @font_dir = split(",",$dir);
    printdebug(1,"reading fonts from directories",@font_dir);
    my @fontrx_pass = split(",",$rx_pass) if $rx_pass;
    my @fontrx_fail = split(",",$rx_fail) if $rx_fail;
    my @search_path;
    my @search_path_root   = (".",
			      $FindBin::Bin,			      
			      $FindBin::RealBin,			      
			      $FindBin::Script);
    my @search_path_suffix = qw(. ..);
    for my $root (@search_path_root) {
	for my $suffix (@search_path_suffix) {
	    for my $dir (@font_dir) {
		push @search_path, sprintf("%s/%s/%s",$root,$suffix,$dir);
	    }
	}
    }
    my @dirs_found;
    for my $dir (@search_path) {
	printdebug(2,"looking for font directory",$dir);
	next unless -d $dir;
	push @dirs_found, $dir;
	printdebug(1,"reading fonts from",$dir);
	opendir(D,$dir);
	while(my $item = readdir(D)) {
	    next if $item eq "." || $item eq "..";
	    next unless $item =~ /(otf|ttf)$/i;
	    next if $rx_fail && grep($item =~ /$_/, @fontrx_fail);
	    if(! defined $rx_pass || grep($item =~ /$_/, @fontrx_pass)) {
		my $font_file = sprintf("%s/%s",$dir,$item);
		printdebug(1,"adding font file",basename($font_file));
		push @font_files, $font_file;
	    }
	}
	return @font_files if @font_files;
    }
    if(! @dirs_found) {
	confess "Could not find font directory anywhere in ",join(" ",@search_path);
    } else {
	confess "Could not find any fonts in directories ",join(" ",@dirs_found);
    }
}

1;
