package ASCII::Glyph;

=pod

=head1 NAME

ASCII::Text - glyph (image from text) handling routines 

=head1 SYNOPSIS

This module is not meant to be used directly.

=head1 DESCRIPTION

=cut

# -------------------------------------------------------------------

use strict;
use warnings;

use base 'Exporter';
our @EXPORT = qw(

);

use Carp qw( carp confess croak );
use Data::Dumper;
use List::MoreUtils qw(uniq);
use FindBin;
use File::Basename;
use File::Path qw(mkpath);
use Math::VecStat qw(sum min max average);

use lib "$FindBin::RealBin";
use lib "$FindBin::RealBin/../lib";
use lib "$FindBin::RealBin/lib";

use ASCII::Debug;
use ASCII::Font;
use ASCII::Image;

sub make_glyphs {
    my $size    = shift;
    my $gsparam = shift;
    my $spacing = shift;
    my @chars   = @_;

    my $scheme = $main::CONF{score}{scheme}{ $main::CONF{score}{scheme_name} };

    my $fg    = make_color($main::CONF{font}{fg},$main::CONF{font}{fga});
    my $bg    = make_color($main::CONF{font}{bg},$main::CONF{font}{bga});

    my @font_files = ASCII::Font::get_font_files($main::CONF{input}{fontdir},
						 $main::CONF{input}{fontrx_pass},
						 $main::CONF{input}{fontrx_fail},
	);
    
    my @glyphs;

    my $max;
    my $num_widths;
    for my $font_file (@font_files) {
	printdebug(1,"collating metrics for font",basename($font_file));
	my $font = ASCII::Font::make_font_obj($font_file,$fg,$size);
	for my $c (uniq @chars) {
	    my %metrics = ASCII::Font::get_metrics($font,$c);
	    printdebug(2,sprintf("metric %s %20s " . "%3d "x8,
				 $c,
				 basename($font_file),
				 @metrics{ @ASCII::Font::metric_names }));
	    $num_widths->{ $metrics{pos_width} }++;
	    if(! defined $max->{ascent} || $metrics{ascent} > $max->{ascent}) {
		$max->{ascent}  = $metrics{ascent};
	    }
	    if(! defined $max->{descent} || $metrics{descent} < $max->{descent}) {
		$max->{descent} = $metrics{descent};
	    }
	}
    }

    for my $font_file (@font_files) {
	printdebug(1,"making glyphs for font",basename($font_file));
	my $font      = ASCII::Font::make_font_obj($font_file,$fg,$size);
	my $font_name = ASCII::Font::get_font_name($font_file);
	for my $c (uniq @chars) {
	    my %metrics = ASCII::Font::get_metrics($font,$c);
	    my $cwidth  = $metrics{pos_width}; 
	    #$cwidth     += $neg_width unless $CONF{font}{fixed} || (keys %$num_widths) == 1;
	    #$cwidth     -= $neg_width if $cwidth <= 0;
	    $cwidth     -= $metrics{neg_width} if $metrics{neg_width} < 0;
	    my $cheight  = $max->{ascent} - $max->{descent};
	    confess "zero size glyph [$c] [$cwidth x $cheight] for font $font_file" if !$cwidth || !$cheight;
	    my $img = Imager->new(xsize=>$cwidth,ysize=>$cheight);
	    confess "problem creating glyph img for character [$c] font [$font_file] cwidth $cwidth" if ! $img;
	    $img->flood_fill("x"=>0,"y"=>0,color=>$bg);
	    die "cannot create imager img " . Imager->errstr() if ! $img;
	    $img->string(font   => $font,
			 text   => $c,
			 "x"    => 0,
			 "y"    => $max->{ascent},
			 size   => $size,
			 color  => $fg,
			 aa     => 1);
	    if($main::CONF{output}{write_glyphs}) {
		my $file = sprintf("%s/%s-%s.png",
				   $main::CONF{output}{glyphdir},
				   $c,
				   basename($font_file));
		mkpath(dirname($file));
		$img->write(file=>$file) || die "cannot write glyph file to [$file] " . $img->errstr();
	    }
	    my $glyph = {c         => $c,
			 tag       => sprintf("%s-%s",$c,basename($font_file)),
			 w         => $cwidth,
			 h         => $cheight,
			 metric    => \%metrics,
			 npix      => $cwidth * $cheight,
			 font_file => $font_file,
			 font_name => $font_name,
			 font      => $font,
			 ascent    => $max->{ascent},
			 descent   => $max->{descent},
			 img       => $img};
	    ASCII::Image::get_pixels($glyph,
				     $gsparam->{glyph_tone_curve},
				     $gsparam->{glyph_signal_cutoff});
				     
	    push @glyphs, $glyph;
	}
    }
    # maximum height of all glyph images
    my $glyph_max_height = max( map {$_->{h}} @glyphs);
    my $glyph_max_width  = max( map {$_->{w}} @glyphs);
    return ($glyph_max_height,
	    $glyph_max_width,
	    @glyphs);
}

1;
