package ASCII::Score;

=pod

=head1 NAME

ASCII::Score - utility routines for scoring character match to image

=head1 SYNOPSIS

This module is not meant to be used directly.

=head1 DESCRIPTION

=cut

# -------------------------------------------------------------------

use strict;
use warnings;

use base 'Exporter';
our @EXPORT = qw();

use Carp qw( carp confess croak );
use Data::Dumper;
use FindBin;

use lib "$FindBin::RealBin";
use lib "$FindBin::RealBin/../lib";
use lib "$FindBin::RealBin/lib";

use ASCII::Debug;
use ASCII::Util;
use ASCII::Timer;

# fitness score (to be minimized) between the glyph and the image
#
# - structure score
# - tone score
# 
sub gscore {
    start_timer("gscore");
    my ($image,$g,$xpos,$ypos,$gsparam,$img_stat) = @_;
    my $score_structure   = 0;
    my $penalty_structure = 0;
    my ($imgw,$imgh)      = @{$image}{qw(w h)};
    my ($img_sum,$img_sum2,$img_n);
    my ($img_min,$img_max) = (1,0);
    my $g_width  = $g->{w};
    my $g_height = $g->{h};
    my $use_sig  = $gsparam->{use_signal_in_structure};
    my $score_match_fg  = $gsparam->{score_match_fg};
    my $score_match_bg  = $gsparam->{score_match_bg};
    my $penalty_nomatch = $gsparam->{penalty_nomatch};
    my $penalty_excess  = $gsparam->{penalty_excess};
    # compute statistics for image region of width g_width, if not already computed
    my $compute_img_stat = ! defined $img_stat->{$g_width};
    # ($gx,$gy) coordinates in glyph
    # ($ix,$iy) coordinates in image
    for my $gx (0..$g_width-1) {
	my $ix      = $gx+$xpos;
	my $gxdata  = $g->{data}[$gx];
	my $ixdata  = $image->{data}[$ix];
	for my $gy (0..$g_height-1) {
	    my $iy      = $gy+$ypos;
	    my $gsignal = $gxdata->[$gy];
	    my $isignal = $ixdata->[$iy];
	    if($compute_img_stat) {
		$img_sum  += $isignal;
		#$img_sum2 += $isignal**$isignal;
		$img_max  = $isignal if $isignal > $img_max;
		$img_min  = $isignal if $isignal < $img_min;
		$img_n++;
	    }
	    if($use_sig) {
		if($isignal) {
		    if($gsignal) {
			# image + glyph +
			$score_structure   += ($isignal * $gsignal) * $score_match_fg;
		    } else {
			# image + glyph -
			$penalty_structure += $isignal * $penalty_nomatch;
		    }
		} else {
		    if($gsignal) {
			# image - glyph +
			$penalty_structure += $gsignal * $penalty_excess;
		    } else {
			# image - glyph -
			$score_structure   += $score_match_bg;
		    }
		}
	    } else {
		if($isignal) {
		    if($gsignal) {
			# image + glyph +
			$score_structure   += $score_match_fg;
		    } else {
			# image + glyph -
			$penalty_structure += $penalty_nomatch;
		    }
		} else {
		    if($gsignal) {
			# image - glyph +
			$penalty_structure += $penalty_excess;
		    } else {
			# image - glyph -
			$score_structure   += $score_match_bg;
		    }
		}
	    }
	}
    }
    #my $img_sd   = 0;
    if($compute_img_stat) {
	$img_stat->{$g_width}{avg}   = $img_n ? $img_sum / $img_n : 0;
	$img_stat->{$g_width}{range} = $img_max - $img_min;
	#$img_sd       = sqrt ( $img_sum2 / $img_n - $img_avg**2 );
	#printdebug(2,"imgstat",$xpos,$ypos,
	#sprintf("avg %.2f min %.2f max %.2f",$img_avg,$img_min,$img_max));
    }
    # seek to minimize tone and structure score
    my $score_tone   = abs($img_stat->{$g_width}{avg} - $g->{avg});
    $score_structure = -$score_structure + $penalty_structure;
    $score_structure /= $g->{npix} if $gsparam->{normalize_by_glyph_size};
    stop_timer("gscore");
    return ($score_structure,$score_tone,
	    $img_stat->{$g_width}{range},
	    $img_stat);
}

sub init_gscore_params {
    my $scheme = shift;
    my $param;
    for my $pname (keys %$scheme,
		   keys %{$main::CONF{score}}) {
	next if exists $param->{$pname};
	my $value = fetch_config($pname,$scheme,$main::CONF{score});
	$param->{$pname} = $value if defined $value;
    }
    return $param;
}

1;
